# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Michael Spencer <spencers1993@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from locale import gettext as _

from gi.repository import Gtk # pylint: disable=E0611
from gi.repository import GObject # pylint: disable=E0611
from gi.repository.Gdk import Color 

import logging
logger = logging.getLogger('contributor_console')

from contributor_console_lib import Window
from contributor_console.AboutContributorConsoleDialog import AboutContributorConsoleDialog
from contributor_console.PreferencesContributorConsoleDialog import PreferencesContributorConsoleDialog

from contributor_console import utils
from contributor_console import launchpad
from contributor_console.view_pane import PreferencesPane

import os, webbrowser, subprocess

import threading

GObject.threads_init()  # all Gtk is in the main thread;
                        # only GObject.idle_add() is in the background thread

# See contributor_console_lib.Window.py for more details about how this class works
class ContributorConsoleWindow(Window):
    __gtype_name__ = "ContributorConsoleWindow"
    
    busy = False
    bugsChecking = False
    codeChecking = False
    
    COLOR_VALID = Color(0, 50000, 0)
    COLOR_INVALID = Color(50000, 0, 0)
    
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(ContributorConsoleWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutContributorConsoleDialog
        self.PreferencesDialog = PreferencesContributorConsoleDialog

        # Code for other initialization actions should be added here.

        utils.window = self
        
        self.ui.panels = PreferencesPane()
        self.ui.panels.add_pane(self.ui.codePane, self.ui.codeIcon, _('Code'))
        self.ui.panels.add_pane(self.ui.bugsPane, self.ui.bugsIcon, _('Bugs'))
        self.ui.panels.add_pane(self.ui.translationsPane, self.ui.translationsIcon, _('Translations'))
        
        self.ui.panels.switch_to_pane(self.ui.codePane)        
        self.ui.vbox1.pack_start(self.ui.panels, True, True, 0)
        self.ui.panels.show()
        
        self.ui.vbox1.pack_start(self.ui.statusBar, True, True, 0)
        self.ui.statusBar.show()
        
        self.ui.panels.panes.set_sensitive(False)
        utils.do_busy(_('Connecting to Launchpad'), launchpad.connect, self.launchpad_initialized)

    ##### ASYNC WRAPPERS THAN ADD STATUS NOTIFICATION #####

    def start_busy(self, msg):
        self.ui.statusBar.set_visible(True)
        self.ui.statusSpinner.start()
        self.ui.statusLabel.set_text(msg)
        
    def end_busy(self):
        self.ui.statusSpinner.stop()
        self.ui.statusLabel.set_text(_('Ready.'))
        self.ui.statusBar.set_visible(False)
        self.busy = False
        
    def done_busy(self, result, error):
        self.end_busy()
        if error:
            utils.error_message(self.ui.statusLabel.get_text(), 
                    _('Unable to finish current operation') + 
                    '\n\n' + str(error))
        self.on_done_busy(result)

    # Function based on code taken from Ask Ubuntu answer
    def switch_to_panel(self, panel):
        # hide all pages
        for child in self.ui.panels.get_children():
            child.hide()
        # show the single page that is active
        panel.show()

    ##### CALLBACKS FOR ASYNC OPERATIONS #####
    
    def launchpad_initialized(self, result):
        self.ui.panels.panes.set_sensitive(True)

    def bugs_finished_identifying(self, result):
        self.ui.bugsPackageEntry.set_text(result)

    def bugs_finished_checking(self, found):
        '''Callback from checking thread'''
        self.bugsChecking = False
        self.ui.bugsCheckButton.set_sensitive(False)
        if found:
            print 'FOUND!!!'
            self.ui.bugsPackageEntry.modify_fg(Gtk.StateFlags.NORMAL, self.COLOR_VALID)
            self.ui.bugsShowButton.set_sensitive(True)
            self.ui.bugsReportButton.set_sensitive(True)
        else:
            self.ui.bugsPackageEntry.modify_fg(Gtk.StateFlags.NORMAL, self.COLOR_INVALID)
            self.ui.bugsShowButton.set_sensitive(False)
            self.ui.bugsReportButton.set_sensitive(False)

    def code_finished_identifying(self, result):
        self.ui.codePackageEntry.set_text(result)
    
    # TODO: Use global application preferences
    def code_finished_branching(self, folder):
        if self.ui.codeTermCheck.get_active():
            subprocess.Popen(['gnome-terminal', folder])
        if self.ui.codeFileCheck.get_active():
            subprocess.Popen(['nautilus', folder])
    
    def code_finished_checking(self, found):
        '''Callback from checking thread'''
        self.codeChecking = False
        self.ui.codeCheckButton.set_sensitive(False)
        if found:
            print 'FOUND!!!'
            self.ui.codePackageEntry.modify_fg(Gtk.StateFlags.NORMAL, self.COLOR_VALID)
            self.ui.codePackageButton.set_sensitive(True)
            self.ui.codeProjectButton.set_sensitive(True)
        else:
            self.ui.codePackageEntry.modify_fg(Gtk.StateFlags.NORMAL, self.COLOR_INVALID)
            self.ui.codePackageButton.set_sensitive(False)
            self.ui.codeProjectButton.set_sensitive(False)
    
    ##### ASYNC TASKS #####
    
    def check_for_package(self, name):
        package = utils.check_for_package(name)
        return package != None
    
    ##### BUTTON AND MENU HANDLERS #####
    
    def on_panel_changed(self, menu_item):
        if menu_item == self.ui.mnu_code:
            self.ui.panels.switch_to_pane(self.ui.codePane)
        elif menu_item == self.ui.mnu_bugs:
            self.ui.panels.switch_to_pane(self.ui.bugsPane)
        elif menu_item == self.ui.mnu_translations:
            self.ui.panels.switch_to_pane(self.ui.translationsPane)
    
    def on_bugsCheckButton_clicked(self, button):
        self.bugsChecking = True
        print 'Checking...'
        self.ui.bugsCheckButton.set_sensitive(False)
        utils.do_busy(_('Checking package name'),
                launchpad.check_for_package,
                self.bugs_finished_checking,
                {'name': self.ui.bugsPackageEntry.get_text()})
    
    def on_bugsShowButton_clicked(self, button):
        url = 'http://bugs.launchpad.net/ubuntu/+source/' + self.ui.bugsPackageEntry.get_text()
        webbrowser.open(url)
        
    def on_bugsReportButton_clicked(self, button):
        subprocess.Popen(['ubuntu-bug', self.ui.bugsPackageEntry.get_text()])
    
    def on_bugsPackageEntry_activate(self, box):
        self.on_bugsCheckButton_clicked(self.ui.bugsCheckButton)
    
    def on_bugsPackageEntry_changed(self, box):
        self.ui.bugsPackageEntry.modify_fg(Gtk.StateFlags.NORMAL, None)
        if (self.bugsChecking):
            self.bugs_cancel_checking()
        if (box.get_text() == ''):
            self.ui.bugsCheckButton.set_sensitive(False)
        else:
            self.ui.bugsCheckButton.set_sensitive(True)
        self.ui.bugsShowButton.set_sensitive(False)
        self.ui.bugsReportButton.set_sensitive(False)
    
    def on_bugsIdentifyButton_clicked(self, button):
        self.identify_package(self.bugs_finished_identifying)
    
    def on_codeCheckButton_clicked(self, button):
        self.codeChecking = True
        print 'Checking...'
        self.ui.codeCheckButton.set_sensitive(False)
        utils.do_busy(_('Checking package name'),
                launchpad.check_for_package,
                self.code_finished_checking,
                {'name': self.ui.codePackageEntry.get_text()})

    def on_codeIdentifyButton_clicked(self, button):
        self.identify_package(self.code_finished_identifying)
    
    def on_codePackageButton_clicked(self, button):
        package = self.ui.codePackageEntry.get_text()
        folder = '/home/mspencer/Projects/' + package
        
        utils.do_busy(_('Branching %s to %s') % (package, folder),
                launchpad.branch_package,
                self.code_finished_branching,
                {'name': package, 'folder': folder})
    
    def on_codePackageEntry_activate(self, box):
        self.on_codeCheckButton_clicked(self.ui.codeCheckButton)
            
    def on_codePackageEntry_changed(self, box):
        self.ui.codePackageEntry.modify_fg(Gtk.StateFlags.NORMAL, None)
        if (self.codeChecking):
            self.code_cancel_checking()
        if (box.get_text() == ''):
            self.ui.codeCheckButton.set_sensitive(False)
        else:
            self.ui.codeCheckButton.set_sensitive(True)
        self.ui.codePackageButton.set_sensitive(False)
        self.ui.codeProjectButton.set_sensitive(False)

    def identify_package(self, on_done):
        # Message taken from Apport
        utils.info_message(_('Identify package'), 
                _('After closing this message please click on an '
                'application window to identify its package.'))
        utils.do_busy('Identifying package',
                utils.identify_package,
                on_done)
