# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Michael Spencer <spencers1993@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from locale import gettext as _

import os, subprocess

from launchpadlib.launchpad import Launchpad
from gi.repository import Gtk, GObject

import threading

import shlex

window = None

# Function based on code taken from Apport
def message_dialog(title, text, _type,
            buttons=Gtk.ButtonsType.OK):
    md = Gtk.MessageDialog(parent=window, message_type=_type, buttons=buttons)
    if 'http://' in text or 'https://' in text:
        if type(text) != type(b''):
            text = text.encode('UTF-8')
        text = GLib.markup_escape_text(text)
        text = re.sub(r'(https?://[a-zA-Z0-9._-]+(?:[a-zA-Z0-9_#?%/-])*)',
                r'<a href="\1">\1</a>', text)
        # turn URLs into links
        md.set_markup(text)
    else:
        # work around gnome #620579
        md.set_property('text', text)
    md.set_title(title)
    result = md.run()
    md.hide()
    while Gtk.events_pending():
        Gtk.main_iteration_do(False)
    md = None
    return result

# Function based on code taken from Apport
def info_message(title, text):
    message_dialog(title, text, Gtk.MessageType.INFO)

# Function based on code taken from Apport
def error_message(title, text):
    message_dialog(title, text, Gtk.MessageType.ERROR)

# Function based on code from Apport
def get_window_pid():
    xprop = subprocess.Popen(['xprop', '_NET_WM_PID'],
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    (out, err) = xprop.communicate()
    if xprop.returncode == 0:
        try:
            return int(out.split()[-1])
        except ValueError:
            error_message(_('Cannot identify package'),
                    _('xprop failed to determine process ID of the window') + '\n\n' + err)
            return -1
    else:
        error_message(_('Cannot identify package'),
                _('xprop failed to determine process ID of the window') + '\n\n' + err)
        return -1

def get_window_exe():
    pid = get_window_pid()
    
    if pid == -1:
        return ''
        
    return os.path.realpath('/proc/' + str(pid) + '/exe')

def identify_package():
    path = get_window_exe()
    
    if path == '':
        return ''
    
    apt_file = subprocess.Popen(['apt-file', 'find', path],
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    (out, err) = apt_file.communicate()
        
    if apt_file.returncode == 0:
        try:
            return out.split()[0][:-1]
        except ValueError:
            error_message(_('Cannot identify package'),
                    _('apt-file failed to determine the package of the window') + '\n\n' + err)
            return ''
    else:
        error_message(_('Cannot identify package'),
                _('apt-file failed to determine the package of the window') + '\n\n' + err)
        return ''

def run(cmd):
    args = shlex.split(cmd)
    result = subprocess.call(args)
    
    if result != 0:
        raise 'Operation returned errorcode ' + str(result)

# Function based on code taken from Ask Ubuntu
def async_call(f, on_done, kargs={}):
    if not on_done:
        on_done = lambda r, e: None

    def do_call():
        result = None
        error = None

        try:
            result = f(**kargs)
        except Exception, err:
            error = err

        GObject.idle_add(lambda: on_done(result, error))
    thread = threading.Thread(target = do_call)
    thread.start()
    
def do_busy(msg, action, on_done, kargs={}):
    if (window.busy):
        return False
    window.busy = True
        
    window.on_done_busy = on_done
    window.start_busy(msg)
    
    async_call(action, window.done_busy, kargs)
    return True
