# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Michael Spencer <spencers1993@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from locale import gettext as _

import cairo

from gi.repository import Gtk, Gdk, Pango, GObject, GdkPixbuf

from contributor_console.view_pane.em import em

def _widget_set_margins(widget, top=0, bottom=0, left=0, right=0):
    widget.set_margin_top(top)
    widget.set_margin_bottom(bottom)
    widget.set_margin_left(left)
    widget.set_margin_right(right)

class GlobalPane(Gtk.Toolbar):

    def __init__(self, view_manager):
        Gtk.Toolbar.__init__(self)
        context = self.get_style_context()
        context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)

        # this is what actually draws the All Software, Installed etc buttons
        self.view_switcher = ViewSwitcher(view_manager)
        self._insert_as_tool_item(self.view_switcher, 1)

        item = Gtk.ToolItem()
        item.set_expand(True)
        self.insert(item, -1)

    def _insert_as_tool_item(self, widget, pos):
        item = Gtk.ToolItem()
        item.add(widget)
        self.insert(item, pos)
        return item

def _update_icon(image, icon, icon_size):
    if isinstance(icon, GdkPixbuf.Pixbuf):
        image = image.set_from_pixbuf(icon)
    elif isinstance(icon, Gtk.Image):
        image = image.set_from_pixbuf(icon.get_pixbuf())
    elif isinstance(icon, str):
        image = image.set_from_icon_name(icon, icon_size)
    elif icon is None:
        image = Gtk.Image()
    else:
        msg = "Acceptable icon values: None, GdkPixbuf, GtkImage or str"
        raise TypeError(msg)
    return image

class ViewSwitcher(Gtk.Box):

    ICON_SIZE = Gtk.IconSize.LARGE_TOOLBAR

    def __init__(self, view_manager):
        # boring stuff
        self.view_manager = view_manager

        # widgetry
        Gtk.Box.__init__(self)
        self.set_orientation(Gtk.Orientation.HORIZONTAL)

        # set sensible atk name
        atk_desc = self.get_accessible()
        atk_desc.set_name(_("Contributor Console"))

    def append_section(self, label, icon):
        btn = SectionSelector(label, icon, self.ICON_SIZE)
        self.pack_start(btn, False, False, 0)

        global _last_button
        if _last_button is not None:
            btn.join_group(_last_button)

        _last_button = btn

        # this must go last otherwise as the buttons are added
        # to the group, toggled & clicked gets emitted... causing
        # all panes to fully initialise on USC startup, which is
        # undesirable!
        btn.connect("button-release-event", self.on_selector_clicked)
        return btn

    def on_selector_clicked(self, button):
        index = self.get_children().index_of(button)
        self.view_manager.switch_to_pane(index)
