# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Michael Spencer <spencers1993@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from locale import gettext as _

import cairo

from gi.repository import Gtk, Gdk, Pango, GObject, GdkPixbuf

class GlobalPane(Gtk.Toolbar):

    def __init__(self, view_manager):
        Gtk.Toolbar.__init__(self)
        context = self.get_style_context()
        context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)

        if self.get_direction() != Gtk.TextDirection.RTL:
            _widget_set_margins(self.back_forward,
                                left=StockEms.MEDIUM,
                                right=StockEms.MEDIUM + 2)
        else:
            _widget_set_margins(self.back_forward,
                                right=StockEms.MEDIUM,
                                left=StockEms.MEDIUM + 2)
        self._insert_as_tool_item(self.back_forward, 0)

        # this is what actually draws the All Software, Installed etc buttons
        self.view_switcher = ViewSwitcher(view_manager)
        self._insert_as_tool_item(self.view_switcher, 1)

        item = Gtk.ToolItem()
        item.set_expand(True)
        self.insert(item, -1)

    def _insert_as_tool_item(self, widget, pos):
        item = Gtk.ToolItem()
        item.add(widget)
        self.insert(item, pos)
        return item

def _update_icon(image, icon, icon_size):
    if isinstance(icon, GdkPixbuf.Pixbuf):
        image = image.set_from_pixbuf(icon)
    elif isinstance(icon, Gtk.Image):
        image = image.set_from_pixbuf(icon.get_pixbuf())
    elif isinstance(icon, str):
        image = image.set_from_icon_name(icon, icon_size)
    elif icon is None:
        image = Gtk.Image()
    else:
        msg = "Acceptable icon values: None, GdkPixbuf, GtkImage or str"
        raise TypeError(msg)
    return image

class _Tile(object):

    MIN_WIDTH = em(7)

    def __init__(self):
        self.set_focus_on_click(False)
        self.set_relief(Gtk.ReliefStyle.NONE)
        self.box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        self.box.set_size_request(self.MIN_WIDTH, -1)
        self.add(self.box)

    def build_default(self, label, icon, icon_size):
        if icon is not None:
            if isinstance(icon, Gtk.Image):
                self.image = icon
            else:
                self.image = Gtk.Image()
                _update_icon(self.image, icon, icon_size)
            self.box.pack_start(self.image, True, True, 0)

        self.label = Gtk.Label.new(label)
        self.box.pack_start(self.label, True, True, 0)

class TileButton(Gtk.Button, _Tile):

    def __init__(self):
        Gtk.Button.__init__(self)
        _Tile.__init__(self)


class TileToggleButton(Gtk.RadioButton, _Tile):

    def __init__(self):
        Gtk.RadioButton.__init__(self)
        self.set_mode(False)
        _Tile.__init__(self)

class ChannelSelector(Gtk.Button):

    PADDING = 0

    def __init__(self, section_button):
        Gtk.Button.__init__(self)
        alignment = Gtk.Alignment.new(0.5, 0.5, 0.0, 1.0)
        alignment.set_padding(self.PADDING, self.PADDING,
                              self.PADDING, self.PADDING)
        self.add(alignment)
        self.arrow = Gtk.Arrow.new(Gtk.ArrowType.DOWN, Gtk.ShadowType.IN)
        alignment.add(self.arrow)

        # vars
        self.parent_style_type = Gtk.Toolbar
        self.section_button = section_button
        self.popup = None
        self.connect("button-press-event", self.on_button_press)

    def do_draw(self, cr):
        cr.save()

        parent_style = self.get_ancestor(self.parent_style_type)
        context = parent_style.get_style_context()

        color = darken(context.get_border_color(Gtk.StateFlags.ACTIVE), 0.2)

        cr.set_line_width(1)

        a = self.get_allocation()
        lin = cairo.LinearGradient(0, 0, 0, a.height)
        lin.add_color_stop_rgba(0.1,
                                color.red,
                                color.green,
                                color.blue,
                                0.0)    # alpha
        lin.add_color_stop_rgba(0.5,
                                color.red,
                                color.green,
                                color.blue,
                                1.0)    # alpha
        lin.add_color_stop_rgba(1.0,
                                color.red,
                                color.green,
                                color.blue,
                                0.1)    # alpha
        cr.set_source(lin)

        cr.move_to(0.5, 0.5)
        cr.rel_line_to(0, a.height)
        cr.stroke()

        cr.move_to(a.width - 0.5, 0.5)
        cr.rel_line_to(0, a.height)
        cr.stroke()

        cr.restore()

        for child in self:
            self.propagate_draw(child, cr)

    def on_button_press(self, button, event):
        if self.popup is None:
            self.build_channel_selector()
        self.show_channel_sel_popup(self, event)
#~
    #~ def on_style_updated(self, widget):
        #~ context = widget.get_style_context()
        #~ context.save()
        #~ context.add_class("menu")
        #~ bgcolor = context.get_background_color(Gtk.StateFlags.NORMAL)
        #~ context.restore()
#~
        #~ self._dark_color = darken(bgcolor, 0.5)

    def show_channel_sel_popup(self, widget, event):

        def position_func(menu, (window, a)):
            if self.get_direction() != Gtk.TextDirection.RTL:
                tmpx = a.x
            else:
                tmpx = a.x + a.width - self.popup.get_allocation().width
            x, y = window.get_root_coords(tmpx,
                                          a.y + a.height)
            return (x, y, False)

        a = self.section_button.get_allocation()
        window = self.section_button.get_window()
        self.popup.popup(None, None, position_func, (window, a),
                         event.button, event.time)

    def set_build_func(self, build_func):
        self.build_func = build_func

    def build_channel_selector(self):
        self.popup = Gtk.Menu()
        self.popup.set_name('toolbar-popup')  # to set 'padding: 0;'
        self.popup.get_style_context().add_class('primary-toolbar')
        self.build_func(self.popup)

class SectionSelector(TileToggleButton):

    MIN_WIDTH = em(5)
    _MARKUP = '<small>%s</small>'

    def __init__(self, label, icon, icon_size=Gtk.IconSize.DIALOG):
        TileToggleButton.__init__(self)
        markup = self._MARKUP % label
        self.build_default(markup, icon, icon_size)
        self.label.set_use_markup(True)
        self.label.set_justify(Gtk.Justification.CENTER)

        context = self.get_style_context()
        context.add_class("section-sel-bg")

        context = self.label.get_style_context()
        context.add_class("section-sel")

        self.draw_hint_has_channel_selector = False
        self._alloc = None
        self._bg_cache = {}

        self.connect('size-allocate', self.on_size_allocate)
        self.connect('style-updated', self.on_style_updated)
        # workaround broken engines (LP: #1021308)
        self.emit("style-updated")

    def on_size_allocate(self, *args):
        alloc = self.get_allocation()

        if (self._alloc is None or
            self._alloc.width != alloc.width or
            self._alloc.height != alloc.height):
            self._alloc = alloc
            # reset the bg cache
            self._bg_cache = {}

    def on_style_updated(self, *args):
        # also reset the bg cache
        self._bg_cache = {}

    def _cache_bg_for_state(self, state):
        a = self.get_allocation()
        # tmp surface on which we render the button bg as per the gtk
        # theme engine
        _surf = cairo.ImageSurface(cairo.FORMAT_ARGB32,
                                   a.width, a.height)
        cr = cairo.Context(_surf)

        context = self.get_style_context()
        context.save()
        context.set_state(state)

        Gtk.render_background(context, cr,
                          -5, -5, a.width + 10, a.height + 10)
        Gtk.render_frame(context, cr,
                          -5, -5, a.width + 10, a.height + 10)
        del cr

        # new surface which will be cached which
        surf = cairo.ImageSurface(cairo.FORMAT_ARGB32,
                                  a.width, a.height)
        cr = cairo.Context(surf)

        # gradient for masking
        lin = cairo.LinearGradient(0, 0, 0, a.height)
        lin.add_color_stop_rgba(0.0, 1, 1, 1, 0.1)
        lin.add_color_stop_rgba(0.25, 1, 1, 1, 0.7)
        lin.add_color_stop_rgba(0.5, 1, 1, 1, 1.0)
        lin.add_color_stop_rgba(0.75, 1, 1, 1, 0.7)
        lin.add_color_stop_rgba(1.0, 1, 1, 1, 0.1)

        cr.set_source_surface(_surf, 0, 0)
        cr.mask(lin)
        del cr

        # cache the resulting surf...
        self._bg_cache[state] = surf

    def do_draw(self, cr):
        cr.save()
        state = self.get_state_flags()
        if self.get_active():
            if state not in self._bg_cache:
                self._cache_bg_for_state(state)

            cr.set_source_surface(self._bg_cache[state], 0, 0)
            cr.paint()
        cr.restore()
        for child in self:
            self.propagate_draw(child, cr)

class ViewManager:
    pass

# Class based on code taken from Software Center
class ViewSwitcher(Gtk.Box):

    ICON_SIZE = Gtk.IconSize.LARGE_TOOLBAR

    def __init__(self, view_manager, desc):
        self.view_manager = view_manager
        
        def on_view_changed(widget, view_id):
            self.view_buttons[view_id].set_active(True)
        self.view_manager.connect('view-changed', on_view_changed)

        # widgetry
        Gtk.Box.__init__(self)
        self.set_orientation(Gtk.Orientation.HORIZONTAL)

        # Gui stuff
        self.view_buttons = {}
        self.selectors = {}
        self._handlers = []

        # set sensible atk name
        atk_desc = self.get_accessible()
        atk_desc.set_name(_(desc))

    def on_section_sel_clicked(self, button, event, view_id):
        vm = self.view_manager

        def config_view():
            # set active pane
            pane = vm.set_active_view(view_id)
            vm.display(pane)
            return False

        self._prev_view = view_id
        GObject.idle_add(config_view)

    def append_section(self, view_id, label, icon):
        btn = SectionSelector(label, icon, self.ICON_SIZE)
        self.view_buttons[view_id] = btn
        self.pack_start(btn, False, False, 0)

        global _last_button
        if _last_button is not None:
            btn.join_group(_last_button)

        _last_button = btn

        # this must go last otherwise as the buttons are added
        # to the group, toggled & clicked gets emitted... causing
        # all panes to fully initialise on USC startup, which is
        # undesirable!
        btn.connect("button-release-event", self.on_section_sel_clicked,
            view_id)
        return btn
